unit FormOpti;

interface

uses
  WinTypes,
  SysUtils, Forms, StdCtrls, Buttons, ExtCtrls, Classes, Controls;

type

  { TfrmArcOption }
  TfrmSortOption = class(TForm)
    grpDirection: TRadioGroup;
    grpSort: TRadioGroup;
    ListBox1: TListBox;
    btnRight: TSpeedButton;
    btnLeft: TSpeedButton;
    ListBox2: TListBox;
    btnApply: TBitBtn;
    Bevel2: TBevel;
    Label1: TLabel;
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ChangeOrderClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckArrows(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MoveToRight(Sender: TObject);
    procedure MoveToLeft(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
  private
    { Private declarations }
  protected
    { Protected declarations }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    procedure SetupWindow(OrderBy: Integer; Ascend: Boolean);
  end;

implementation

{$R *.DFM}

uses
  SDIMain;

procedure TfrmSortOption.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    WndParent := TForm(Owner).Handle;
    Style := Style or ws_DlgFrame;
  end;
end;

procedure TfrmSortOption.SetupWindow(OrderBy: Integer; Ascend: Boolean);
begin
  grpSort.ItemIndex := OrderBy;
  grpDirection.ItemIndex := Word(not Ascend);
  grpDirection.OnClick := ChangeOrderClick;
  grpSort.OnClick := ChangeOrderClick;
end;

procedure TfrmSortOption.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close;
end;

procedure TfrmSortOption.ChangeOrderClick(Sender: TObject);
begin
  SDIAppForm.List.ReorderBy(grpSort.ItemIndex, grpDirection.ItemIndex=0);
  SDIAppForm.UpdateListBox;
end;

procedure TfrmSortOption.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmSortOption.FormCreate(Sender: TObject);
begin
  CheckArrows(nil);
end;

procedure TfrmSortOption.CheckArrows(Sender: TObject);
begin
  btnRight.Enabled := (ListBox1.ItemIndex <> -1) and (ListBox1.Items.Count > 0);
  btnLeft.Enabled := (ListBox2.ItemIndex <> -1) and (ListBox2.Items.Count > 0);
  btnApply.Enabled := ListBox2.Items.Count > 0;
end;

procedure TfrmSortOption.MoveToRight(Sender: TObject);
begin
  ListBox2.Items.Add(ListBox1.Items[ListBox1.ItemIndex]);
  ListBox1.Items.Delete(ListBox1.ItemIndex);
  CheckArrows(Sender);
end;

procedure TfrmSortOption.MoveToLeft(Sender: TObject);
begin
  ListBox1.Items.Add(ListBox2.Items[ListBox2.ItemIndex]);
  ListBox2.Items.Delete(ListBox2.ItemIndex);
  CheckArrows(Sender);
end;

procedure TfrmSortOption.btnApplyClick(Sender: TObject);
var
  Keys: array [0..4] of Longint;
  I: Integer;
begin
  for I := 0 to 4 do Keys[I] := -1;
  for I := 0 to ListBox2.Items.Count - 1 do
    if ListBox2.Items[I] = 'Name' then Keys[I] := 0 else
    if ListBox2.Items[I] = 'Phone' then Keys[I] := 1 else
    if ListBox2.Items[I] = 'EMail' then Keys[I] := 2 else
    if ListBox2.Items[I] = 'Web Site' then Keys[I] := 3 else
    if ListBox2.Items[I] = 'Age' then Keys[I] := 4;
  SDIAppForm.List.ReorderByKeys(grpDirection.ItemIndex=0, Keys);
  SDIAppForm.UpdateListBox;
end;

end.

