unit SDIMain;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Classes, Graphics, Forms, Dialogs, Controls, Menus,
  StdCtrls, Buttons, ExtCtrls, SomeType;

type

  { TSDIAppForm }
  TSDIAppForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    OpenItem: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Help1: TMenuItem;
    About1: TMenuItem;
    StatusBar: TPanel;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    Header1: THeader;
    ListBox1: TListBox;
    Create1: TMenuItem;
    btnAdd: TSpeedButton;
    btnDel: TSpeedButton;
    btnEdit: TSpeedButton;
    Options1: TMenuItem;
    Search1: TMenuItem;
    chkSave: TCheckBox;
    Registration1: TMenuItem;
    procedure ShowHint(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure OpenItemClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Header1Sized(Sender: TObject; ASection, AWidth: Integer);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure Create1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnEditClick(Sender: TObject);
    procedure Options1Click(Sender: TObject);
    procedure Search1Click(Sender: TObject);
    procedure Registration1Click(Sender: TObject);
  private
    { Private declarations }
    Modified: Boolean;
    procedure SaveList;
  public
    { Public declarations }
    List: TClientCollection;
    procedure UpdateListBox;
    procedure UpdateMenu;
    procedure UpdateListBoxIndex(NewIndex: Longint);
  end;

var
  SDIAppForm: TSDIAppForm;

implementation

uses
  About, OrdInfo, FormOpti, FindStuf, CliSheet;

{$R *.DFM}

procedure TSDIAppForm.ShowHint(Sender: TObject);
begin
  StatusBar.Caption := Application.Hint;
end;

procedure TSDIAppForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TSDIAppForm.About1Click(Sender: TObject);
begin
  ShowAboutBox(Self);
end;

procedure TSDIAppForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  List := nil;
  Modified := False;
end;

procedure TSDIAppForm.FormDestroy(Sender: TObject);
begin
  if Assigned(List) then List.Free;
end;

procedure TSDIAppForm.FormShow(Sender: TObject);
begin
  with Header1 do
  begin
    SectionWidth[0] := 130;
    SectionWidth[1] := 100;
    SectionWidth[2] := 130;
    SectionWidth[3] := 185;
    SectionWidth[4] := 80;
  end;
end;

procedure TSDIAppForm.SaveList;
begin
  if Assigned(List) and Modified then
    if Application.MessageBox('Save Client List?', 'Confirm', MB_ICONQUESTION or MB_YESNO) = IDYES then
      if SaveDialog.Execute then
      begin
        StoreClientCollection(SaveDialog.FileName, List);
        Modified := False;
      end;
end;

procedure TSDIAppForm.Create1Click(Sender: TObject);
begin
  SaveList;
  List := CreateSampleClientCollection(True);
  UpdateListBox;
  Modified := False;
end;

procedure TSDIAppForm.OpenItemClick(Sender: TObject);
begin
  SaveList;
  if OpenDialog.Execute then
  begin
    if Assigned(List) then List.Free;
    List := LoadClientCollection(OpenDialog.FileName);
    UpdateListBox;
    Modified := False;
  end;
end;

procedure TSDIAppForm.btnDelClick(Sender: TObject);
begin
  if Assigned(List) and (List.Count > 0) and (ListBox1.ItemIndex <> -1) then
    with TClient(List.Items[ListBox1.ItemIndex]) do
      if Application.MessageBox('Selected client record is about'#10'to be DELETED!'#10'Proceed?',
	'Confirm', MB_ICONQUESTION or MB_YESNO)= IDYES then
      begin
	List.FreeAtIndex(ListBox1.ItemIndex);
	UpdateListBox;
        Modified := True;
      end;
end;

procedure TSDIAppForm.Header1Sized(Sender: TObject; ASection, AWidth: Integer);
begin
  ListBox1.Invalidate;
end;

procedure TSDIAppForm.UpdateListBox;
var
  I, Old: Integer;
begin
  Old := ListBox1.ItemIndex;
  ListBox1.Clear;
  if Assigned(List) and (List.Count > 0) then
    for I := 0 to List.Count - 1 do
      ListBox1.Items.Add('item');
  if Old = ListBox1.Items.Count then
    Old := ListBox1.Items.Count - 1;
  ListBox1.ItemIndex := Old;
  ListBox1.Invalidate;
  UpdateMenu;
end;

procedure TSDIAppForm.ListBox1DrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Client: TClient;
  R: TRect;
begin
  ListBox1.Canvas.FillRect(Rect);
  if not Assigned(List) or (List.Count = 0) then Exit;

  Client := List.Items[Index];
  R := Rect;
  R.Left := 3;
  R.Right := Header1.SectionWidth[0] - 3;
  ListBox1.Canvas.TextOut(R.Left, R.Top, Client.Name);

  Inc(R.Left, Header1.SectionWidth[0]);
  R.Right := R.Left + Header1.SectionWidth[1] - 6;
  ListBox1.Canvas.TextOut(R.Left, R.Top, Client.Phone);

  Inc(R.Left, Header1.SectionWidth[1]);
  R.Right := R.Left + Header1.SectionWidth[2] - 6;
  ListBox1.Canvas.TextOut(R.Left, R.Top, Client.EMail);

  Inc(R.Left, Header1.SectionWidth[2]);
  R.Right := R.Left + Header1.SectionWidth[3] - 6;
  ListBox1.Canvas.TextOut(R.Left, R.Top, Client.WebSite);

  Inc(R.Left, Header1.SectionWidth[3]);
  R.Right := R.Left + Header1.SectionWidth[4] - 6;
  ListBox1.Canvas.TextOut(R.Left, R.Top, IntToStr(Client.Age));
end;

procedure TSDIAppForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  Answer: Integer;
begin
  CanClose := not (Modified or chkSave.Checked);
  if not CanClose then
  begin
    Answer := Application.MessageBox('Save Client List?', 'Confirm', MB_ICONQUESTION or MB_YESNOCANCEL);
    if Answer = IDYES then
    begin
      SaveDialog.FileName := OpenDialog.FileName;
      if SaveDialog.Execute then
        StoreClientCollection(SaveDialog.FileName, List);
      CanClose := True;
    end
    else
      CanClose := Answer <> IDCANCEL;
  end;
end;

procedure TSDIAppForm.btnAddClick(Sender: TObject);
var
  Client: TClient;
begin
  Client := GetNewClient(Self);
  if Client <> nil then
  begin
    if not Assigned(List) then
      List := TClientCollection.Create(10, 4);
    List.InsertItem(Client);
    UpdateListBox;
    Modified := True;
  end;
end;

procedure TSDIAppForm.btnEditClick(Sender: TObject);
var
  Client: TClient;
begin
  if Assigned(List) and (List.Count > 0) and (ListBox1.ItemIndex <> -1) then
  begin
    Client := TClient.Clone(List.Items[ListBox1.ItemIndex]);
    if not EditNewClient(Self, Client) then Client.Free
    else
    begin
      List.FreeAtIndex(ListBox1.ItemIndex);
      List.InsertItem(Client);
      UpdateListBox;
      Modified := True;
    end;
  end;
end;

procedure TSDIAppForm.Options1Click(Sender: TObject);
begin
  with TfrmSortOption.Create(Self) do
  try
    SetupWindow(List.OrderBy, List.Ascending);
    ShowModal;
  finally
    Free;
  end;
end;

procedure TSDIAppForm.Search1Click(Sender: TObject);
begin
  ExecQuery(Self, List);
  Search1.Enabled := False;
end;

procedure TSDIAppForm.UpdateMenu;
begin
  Options1.Enabled := Assigned(List);
  Search1.Enabled := Assigned(List);
end;

procedure TSDIAppForm.UpdateListBoxIndex(NewIndex: Longint);
begin
  ListBox1.ItemIndex := NewIndex;
end;

procedure TSDIAppForm.Registration1Click(Sender: TObject);
begin
  Show_formOrdInfo(Self);
end;

end.

