unit Sometype;

interface

uses
  SysUtils, Classes, LongColl;

const

  ID_FIRST_OBJECTTYPE = 10000;

{ TClient ------------------------------------------------------ }
const

  ObjectTypeClient = ID_FIRST_OBJECTTYPE + 0;
  VersionClient = $0001;

type

  { TClient }
  TClient = class(TDStreamableClass)
  private
    { Private declarations }
    FName   : String;
    FPhone  : String;
    FEMail  : String;
    FWebSite: String;
    FAge    : Longint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AName, APhone, AEMail, AWebSite: String; AAge: Longint);
    constructor Clone(const C: TDStreamableClass); override;
    constructor Load(S: TDStream); override;
    procedure Store(S: TDStream); override;
    function CompareAge(AAge: Longint): Longint;

    property Name: String read FName write FName;
    property Phone: String read FPhone write FPhone;
    property EMail: String read FEMail write FEMail;
    property WebSite: String read FWebSite write FWebSite;
    property Age: Longint read FAge write FAge;
  end;

{ TClientCollection ------------------------------------------------------ }
const

  ObjectTypeClientCollection = ID_FIRST_OBJECTTYPE + 1;
  VersionClientCollection = $0001;

type

  { TClientCollection }
  TClientCollection = class(TReorderCollection)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure FreeItem(Item: Pointer); override;
    function Compare(Key1, Key2: Pointer): Longint; override;
    function KeyOf(Item: Pointer): Pointer; override;
    procedure ReorderBy(AOrderBy: Byte; AAscending: Boolean); override;
    procedure ReorderByKeys(AAscending: Boolean; const Keys: array of Longint); override;
  end;

function CreateSampleClientCollection(StoreIt: Boolean): TClientCollection;
procedure StoreClientCollection(FileName: String; AList: TClientCollection);
function LoadClientCollection(FileName: String): TClientCollection;

implementation

var
  RClient,
  RClientCollection: TStreamRec;

procedure RegisterThisUnit;
begin
  RegisterStreamObject(RClient, ObjectTypeClient, VersionClient,
    Pointer(TClient), @TClient.Load, @TClient.Store, True);
  RegisterStreamObject(RClientCollection, ObjectTypeClientCollection, VersionClientCollection,
    Pointer(TClientCollection), @TClientCollection.Load, @TClientCollection.Store, False);
end;

{ TClient ------------------------------------------------------ }
constructor TClient.Create(AName, APhone, AEMail, AWebSite: String; AAge: Longint);
begin
  inherited Create;
  FName := AName;
  FPhone := APhone;
  FEMail := AEMail;
  FWebSite := AWebSite;
  FAge := AAge;
end;

constructor TClient.Clone(const C: TDStreamableClass);
begin
  inherited Clone(C);
  if C is TClient then
  begin
    FName    := TClient(C).Name;
    FPhone   := TClient(C).Phone;
    FEMail   := TClient(C).EMail;
    FWebSite := TClient(C).WebSite;
    FAge     := TClient(C).Age;
  end;
end;

constructor TClient.Load(S: TDStream);
begin
  inherited Load(S);
  S.ReadStrArr([Longint(@FName), Longint(@FPhone), Longint(@FEMail), Longint(@FWebSite)]);
  S.Read(FAge, SizeOf(FAge));
  LoadDone(TClient);
end;

procedure TClient.Store(S: TDStream);
begin
  inherited Store(S);
  S.WriteStrArr([FName, FPhone, FEMail, FWebSite]);
  S.Write(FAge, SizeOf(FAge));
  StoreDone(S, TClient);
end;

function TClient.CompareAge(AAge: Longint): Longint;
begin
  Result := FAge - AAge;
end;

{ far functions used by TClientCollection to map fields for sorting }

function CompareName(Key1, Key2: Pointer): LongInt; far;
begin
  Result := CompareStr(String(Key1^), String(Key2^));
end;

function CompareLong(Key1, Key2: Pointer): LongInt; far;
begin
  Result := Longint(Key1^)- Longint(Key2^);
end;

function Compare_Name(Key1, Key2: Pointer): LongInt; far;
begin
  Result := CompareStr(TClient(Key1).Name, TClient(Key2).Name);
end;

function Compare_Phone(Key1, Key2: Pointer): LongInt; far;
begin
  Result := CompareStr(TClient(Key1).Phone, TClient(Key2).Phone);
end;

function Compare_EMail(Key1, Key2: Pointer): LongInt; far;
begin
  Result := CompareStr(TClient(Key1).EMail, TClient(Key2).EMail);
end;

function Compare_WebSite(Key1, Key2: Pointer): LongInt; far;
begin
  Result := CompareStr(TClient(Key1).WebSite, TClient(Key2).WebSite);
end;

function Compare_Age(Key1, Key2: Pointer): LongInt; far;
begin
  Result := TClient(Key1).Age - TClient(Key2).Age;
end;

function KeySelf(Item: Pointer): Pointer; far;
begin
  Result := Item;
end;

function KeyName(Item: Pointer): Pointer; far;
begin
  Result := @TClient(Item).FName;
end;

function KeyPhone(Item: Pointer): Pointer; far;
begin
  Result := @TClient(Item).FPhone;
end;

function KeyEMail(Item: Pointer): Pointer; far;
begin
  Result := @TClient(Item).FEMail;
end;

function KeyWebSite(Item: Pointer): Pointer; far;
begin
  Result := @TClient(Item).FWebSite;
end;

function KeyAge(Item: Pointer): Pointer; far;
begin
  Result := @TClient(Item).FAge;
end;

function CompareOnKeys(Key1, Key2: Pointer; const Keys: array of Longint): Longint; far;
var
  I: Integer;
  Func: TCompareFunc;
begin
  for I := Low(Keys) to High(Keys) do
  begin
    if Keys[I] < 0 then Break; {!!}
    case Keys[I] of
      0: Func := Compare_Name;
      1: Func := Compare_Phone;
      2: Func := Compare_EMail;
      3: Func := Compare_WebSite;
      4: Func := Compare_Age;
    end;
    Result := Func(Key1, Key2);
    if Result <> 0 then Break;
  end;
end;

{ TClientCollection ------------------------------------------------------ }
procedure TClientCollection.FreeItem(Item: Pointer);
begin
  if Assigned(Item) then
    TClient(Item).Free;
end;

function TClientCollection.Compare(Key1, Key2: Pointer): Longint;
begin
  Result := CompareName(Key1, Key2);
end;

function TClientCollection.KeyOf(Item: Pointer): Pointer;
begin
  Result := KeyName(Item);
end;

procedure TClientCollection.ReorderBy(AOrderBy: Byte; AAscending: Boolean);
begin
  inherited ReorderBy(AOrderBy, AAscending);
  case AOrderBy of
    0: Reorder(KeyName,    CompareName);
    1: Reorder(KeyPhone,   CompareName);
    2: Reorder(KeyEMail,   CompareName);
    3: Reorder(KeyWebSite, CompareName);
    4: Reorder(KeyAge,     CompareLong);
  end;
end;

procedure TClientCollection.ReorderByKeys(AAscending: Boolean;
  const Keys: array of Longint);
begin
  inherited ReorderByKeys(AAscending, Keys);
  ReorderOnKeys(KeySelf, CompareOnKeys, Keys);
end;

function CreateSampleClientCollection(StoreIt: Boolean): TClientCollection;
begin
  Result := TClientCollection.Create(10, 4);
  Result.InsertItem(TClient.Create
    ('John Smith','(212) 123-4567','sjohn@aolew.com','http://aw.some.net/~sjohn',15));
  Result.InsertItem(TClient.Create
    ('Aldous Yanskz','(489) 433-4533','yaldous@aolew.com','http://www.awsome.com/~yaldous',30));
  Result.InsertItem(TClient.Create
    ('Morris Bruebacker','(302) 201-9876','bmorris@aolew.com','http://free.email.com/~bmorris',20));
  if StoreIt then
    StoreClientCollection('sample.dat', Result);
end;

procedure StoreClientCollection(FileName: String; AList: TClientCollection);
begin
  with TDStream.Create(FileName, fmCreate, 8192) do
  try
    Put(AList);
  finally
    Free;
  end;
end;

function LoadClientCollection(FileName: String): TClientCollection;
begin
  with TDStream.Create(FileName, fmOpenRead, 8192) do
  try
    Result := TClientCollection(Get);
  finally
    Free;
  end;
end;

initialization
  RegisterThisUnit;
end.

The information provided above is provided "AS IS" without warranty of any kind.
The AUTHOR disclaims all warranties, either express or implied, including the
warranties of merchantability and fitness for a particular purpose.

In no event shall the AUTHOR or his suppliers be liable for any damages whatsoever
including direct, indirect, incidental, consequential, loss of business profits or
special damages, even if the AUTHOR or his suppliers have been advised of the
possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

