{$IFDEF WIN32}
  {$R STRINGS.R32}
  {$R EXPTBMPS.R32}
  {$R VER32.RES}
{$ELSE}
  {$R STRINGS.R16}
  {$R EXPTBMPS.R16}
  {$R VER16.RES}
{$ENDIF}

type

  TClassExpert = class(TIExpert)
    function GetName: string; override;
    {$IFNDEF WIN32}
    function GetGlyph: HBITMAP; override;
    {$ENDIF}
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    {$IFDEF WIN32}
    function GetAuthor: string; override;
    {$ENDIF}
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

procedure HandleException;
begin
  ToolServices.RaiseException(ReleaseException);
end;

{ TClassExpert }
function TClassExpert.GetName: string;
begin
  try
    Result := LoadStr(sClassExpertName);
  except
    HandleException;
  end;
end;

{$IFNDEF WIN32}
function TClassExpert.GetGlyph: HBITMAP;
begin
  try
    Result := LoadBitmap(HInstance, 'CLASSEXPT');
  except
    HandleException;
  end;
end;
{$ENDIF}

function TClassExpert.GetMenuText: string;
begin
  try
    Result := 'Class Expert';
  except
    HandleException;
  end;
end;

function TClassExpert.GetStyle: TExpertStyle;
begin
  try
    Result := esStandard;
  except
    HandleException;
  end;
end;

function TClassExpert.GetState: TExpertState;
begin
  try
    Result := [esEnabled];
  except
    HandleException;
  end;
end;

function TClassExpert.GetIDString: string;
begin
  try
    Result := 'Jee.ClassExpert';
  except
    HandleException;
  end;
end;

{$IFDEF WIN32}
function TClassExpert.GetAuthor: string;
begin
  try
    Result := 'Jee Software';
  except
    HandleException;
  end;
end;
{$ENDIF}

procedure TClassExpert.Execute;
begin
  try
    DialogExpert(ToolServices);
  except
    HandleException;
  end;
end;

procedure DoneExpert; export;
begin
end;

function InitExpert(ToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; export;
  {$IFDEF WIN32}stdcall;{$ENDIF}

procedure DoIt;
begin
  ExptIntf.ToolServices := ToolServices;
  if ToolServices <> nil then
    Application.Handle := ToolServices.GetParentHandle;
end;

begin
  {$IFDEF WIN32}
  if ExptIntf.ToolServices = nil then
  {$ELSE}
  Result := ExptIntf.ToolServices = nil;
  if not Result then Exit
  else
  {$ENDIF}
  DoIt;
  Terminate := DoneExpert;
  Result := RegisterProc(TClassExpert.Create);
end;

{$IFNDEF WIN32}
function FaultHandler(FaultID: Word; FaultAddr: Pointer): TFaultResponse; export;
begin
  DefaultExceptHandler(FaultId, FaultAddr);
end;
{$ENDIF}

exports
  {$IFNDEF WIN32}FaultHandler name FaultHandlerSignature resident,{$ENDIF}
  InitExpert name ExpertEntryPoint resident;
