unit Dlg;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Menus,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, ToolIntf,
  LongColl, RecTypes, Spin;

type

  TDlgExpert = class(TForm)
    btnCreate: TBitBtn;
    BitBtn3: TBitBtn;
    Label1: TLabel;
    edtClassName: TEdit;
    Label2: TLabel;
    cbxAncestor: TComboBox;
    lbx1: TListBox;
    Bevel1: TBevel;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lbx2: TListBox;
    btnfAdd: TBitBtn;
    btnfEdit: TBitBtn;
    btnfDelete: TBitBtn;
    btnAdd: TBitBtn;
    btnDel: TBitBtn;
    Label6: TLabel;
    cbxItemType: TComboBox;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    btnSave: TButton;
    btnLoad: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Label7: TLabel;
    edtFirstObject: TSpinEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure btnfAddClick(Sender: TObject);
    procedure edtClassNameChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure lbx1Click(Sender: TObject);
    procedure lbx2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnfDeleteClick(Sender: TObject);
    procedure cbxItemTypeExit(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure UpDownClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    DrawBitmap: TBitmap;
    SourceBuffer: PChar;
    WorkList: TClassList;
    procedure FmtWrite(Stream: TStream; Fmt: PChar; const Args: array of const);
    function CreateSource(const UnitIdent: string): TMemoryStream;
    function CurrentClass: TClassItem;
    function CurrentField: TClassField;
    procedure Resetlbx2;
    procedure EnableItemsType(Enable: Boolean);
  public
    { Public declarations }
  end;

procedure DialogExpert(ToolServices: TIToolServices);

implementation

uses
  TabNotBk, Tabs, Proxies, VirtIntf, IStreams, ExConst, Fields;

{$R *.DFM}

const
  PgOne = 0;
  PgTwo = 1;

  SourceBufferSize = 1024;

{ TDlgExpert }
procedure TDlgExpert.EnableItemsType(Enable: Boolean);
begin
  cbxItemType.Enabled := Enable;
  if Enable then
    cbxItemType.Color := clWindow
  else
    cbxItemType.Color := clBtnFace;
end;

procedure TDlgExpert.FormShow(Sender: TObject);
var
  NoTabs: Integer;
  Tabs: array [0..1] of Integer;
begin
  NoTabs := 2;
  Tabs[0] := 75;
  Tabs[1] := 142;
  SendMessage(lbx2.Handle, lb_SetTabStops, NoTabs, Longint(@Tabs));
end;

procedure TDlgExpert.FormCreate(Sender: TObject);
begin
  {$IFDEF WIN32}
  lbx2.TabWidth := 10;
  {$ENDIF}
  DrawBitmap := TBitmap.Create;
  cbxAncestor.ItemIndex := 0;
  btnCreate.Caption := LoadStr(sCreate);
  btnCreate.Glyph.Handle := LoadBitmap(HInstance, 'CREATE');
  btnCreate.NumGlyphs := 1;
  RegisteringEnabled := False;
  WorkList := TClassList.Create(5, 2);
  edtClassNameChange(edtClassName);
end;

procedure TDlgExpert.FormDestroy(Sender: TObject);
begin
  WorkList.Free;
  DrawBitmap.Free;
end;

procedure TDlgExpert.CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TDlgExpert.FmtWrite(Stream: TStream; Fmt: PChar;
  const Args: array of const);
begin
  if (Stream <> nil) and (SourceBuffer <> nil) then
  begin
    StrLFmt(SourceBuffer, SourceBufferSize, Fmt, Args);
    Stream.Write(SourceBuffer[0], StrLen(SourceBuffer));
  end;
end;

function TDlgExpert.CreateSource(const UnitIdent: string): TMemoryStream;
const
  CRLF  = #13#10;
  CRLF2 = CRLF+CRLF;

procedure WriteClassInterface(C: TClassItem); far;
var
  LenToAdd: Integer;

procedure WriteField(F: TClassField); far;
var
  FmtString: String;
begin
  if F.FType < trObject then
    FmtString := '    F%-' + Format('%ds: ', [LenToAdd]) + '%s;' + CRLF + #0
  else
    FmtString := '    F%-' + Format('%ds: ', [LenToAdd])+ 'T%s;' + CRLF + #0;

  FmtWrite(Result,
    @FmtString[1],
    [F.FieldName, F.FieldType]);
end;

procedure WriteFieldAsProp(F: TClassField); far;
begin
  with F do
    if FieldPublic then
      if FType < trObject then
        FmtWrite(Result,
          '    property %s: %s read F%0:s write F%0:s;' + CRLF,
          [FieldName, FieldType])
      else
        FmtWrite(Result,
          '    property %s: T%s read F%0:s write F%0:s;' + CRLF,
          [FieldName, FieldType]);
end;

procedure WriteCreate(F: TClassField); far;
begin
  with F do
  begin
    if Owner.Curr > 0 then
      FmtWrite(Result, '      ', [nil]);
    if FType < trObject then
      FmtWrite(Result, 'A%s: %s', [FieldName, FieldType])
    else
      FmtWrite(Result, 'A%s: T%s', [FieldName, FieldType]);
    if Owner.Curr < Owner.Count - 1 then
      FmtWrite(Result, ';' + CRLF, [nil])
    else
      FmtWrite(Result, ');' + CRLF, [nil]);
  end;
end;

begin
  with C do
  begin
    if Fields <> nil then
      LenToAdd := Fields.GetMaxNameLength
    else
      LenToAdd := 2;
    FmtWrite(Result,
      '{ T%s ------------------------------------------------------ }' + CRLF +
      'const' + CRLF2 +
      '  OBJTYPE_%0:s = ID_FIRST_OBJTYPE + %d;' + CRLF +
      '  VERSION_%0:s = $0001;' + CRLF2, [Name, Owner.Curr]);

    FmtWrite(Result,
      'type' + CRLF2 +
      '  { T%s }' + CRLF +
      '  T%0:s = class(%s)' + CRLF, [Name, Ancestor]);
    FmtWrite(Result,
      '  private' + CRLF +
      '    { Private declarations }' + CRLF,
      [nil]);

    if Fields <> nil then
      Fields.ForEach(@WriteField);

    if (Fields <> nil) and (Fields.Count > 0) then
      FmtWrite(Result,
        '    destructor Destroy; override;' + CRLF,
        [nil]);

    FmtWrite(Result,
      '  protected' + CRLF +
      '    { Protected declarations }' + CRLF +
      '    constructor Load(S: TDStream); override;' + CRLF +
      '    procedure Store(S: TDStream); override;' + CRLF,
      [nil]);
    if Collection then
      FmtWrite(Result,
	'    procedure FreeItem(Item: Pointer); override;' + CRLF,
        [nil]);

    if SortedCollection then
      FmtWrite(Result,
        '    function Compare(Key1, Key2: Pointer): Longint; override;' + CRLF +
        '    function KeyOf(Item: Pointer): Pointer; override;' + CRLF,
        [nil]);

    FmtWrite(Result,
      '  public' + CRLF +
      '    { Public declarations }' + CRLF,
      [nil]);

    if (Fields <> nil) and (Fields.Count > 0) and not Collection then
    begin
      FmtWrite(Result,
        '    constructor Create(',
        [nil]);
      Fields.ForEach(@WriteCreate);
    end;

    if Fields <> nil then
      Fields.ForEach(@WriteFieldAsProp);

    { end of class declaration }
    FmtWrite(Result,
      '  end;' + CRLF2,
      [nil]);
  end;
end;

procedure WriteStreamRecs(C: TClassItem); far;
begin
  with C do
  begin
    FmtWrite(Result, '  R%s', [Name]);
    if Owner.Curr < Owner.Count - 1 then
      FmtWrite(Result, ',' + CRLF, [nil])
    else
      FmtWrite(Result, ': TStreamRec;' + CRLF2, [nil]);
  end;
end;

procedure WriteFieldImplementation(C: TClassItem); far;

procedure WriteCreate(F: TClassField); far;
begin
  with F do
  begin
    if Owner.Curr > 0 then FmtWrite(Result, '  ', [nil]);
    if FType < trObject then
      FmtWrite(Result, 'A%s: %s', [FieldName, FieldType])
    else
      FmtWrite(Result, 'A%s: T%s', [FieldName, FieldType]);
    if Owner.Curr < Owner.Count - 1 then
      FmtWrite(Result, ';' + CRLF, [nil])
    else
      FmtWrite(Result, ');' + CRLF, [nil]);
  end;
end;

procedure WriteAssign(F: TClassField); far;
begin
  with F do
    FmtWrite(Result, '  F%s := A%0:s;' + CRLF, [FieldName]);
end;

procedure WriteLoad(F: TClassField); far;
begin
  with F do
    case FType of
        trString: FmtWrite(Result, '  F%s := S.ReadStr;' + CRLF, [FieldName]);
         trPChar: FmtWrite(Result, '  F%s := S.StrRead;' + CRLF, [FieldName]);
          trChar: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trBoolean: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trShortint: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trSmallint: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trLongint: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trByte: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trWord: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trInteger: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trCardinal: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trReal: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trSingle: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trDouble: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trExtended: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trComp: FmtWrite(Result, '  S.Read(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trObject: FmtWrite(Result, '  F%s := T%s(S.Get);' + CRLF, [FieldName, FieldType]);
    end;
end;

procedure WriteStore(F: TClassField); far;
begin
  with F do
    case FType of
        trString: FmtWrite(Result, '  S.WriteStr(F%s);' + CRLF, [FieldName]);
         trPChar: FmtWrite(Result, '  S.StrWrite(F%s);' + CRLF, [FieldName]);
          trChar: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trBoolean: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trShortint: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trSmallint: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trLongint: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trByte: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trWord: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
       trInteger: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trCardinal: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trReal: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trSingle: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trDouble: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
      trExtended: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
          trComp: FmtWrite(Result, '  S.Write(F%s, SizeOf(%s));' + CRLF, [FieldName, FieldType]);
        trObject: FmtWrite(Result, '  S.Put(F%s);' + CRLF, [FieldName]);
    end;
end;

begin
  with C do
  begin
    FmtWrite(Result,
      '{ T%s ------------------------------------------------------ }' + CRLF,
      [Name]);
    if (Fields <> nil) and (Fields.Count > 0) then
    begin
      if not Collection then
      begin
        FmtWrite(Result,
          'constructor T%s.Create(', [Name]);
        if Fields <> nil then Fields.ForEach(@WriteCreate);
        FmtWrite(Result,
          'begin' + CRLF +
          '  inherited Create;' + CRLF,
          [nil]);
        if Fields <> nil then Fields.ForEach(@WriteAssign);
        FmtWrite(Result,
          '  { your code here }' + CRLF +
          'end;' + CRLF2,
          [nil]);
      end;
      FmtWrite(Result,
        'destructor T%s.Destroy;' + CRLF +
        'begin' + CRLF +
        '  { your code here }' + CRLF +
        '  inherited Destroy;' + CRLF +
        'end;' + CRLF2,
        [Name]);
    end;

    FmtWrite(Result,
      'constructor T%s.Load(S: TDStream);' + CRLF +
      'begin' + CRLF +
      '  inherited Load(S);' + CRLF,
      [Name]);
    if Fields <> nil then Fields.ForEach(@WriteLoad);
    FmtWrite(Result,
      '  LoadDone(T%s);' + CRLF +
      'end;' + CRLF2, [Name]);
    FmtWrite(Result,
      'procedure T%s.Store(S: TDStream);' + CRLF +
      'begin' + CRLF +
      '  inherited Store(S);' + CRLF,
      [Name]);
    if Fields <> nil then Fields.ForEach(@WriteStore);
    FmtWrite(Result,
      '  StoreDone(S, T%s);' + CRLF +
      'end;' + CRLF2, [Name]);
    if Collection then
      FmtWrite(Result,
        'procedure T%s.FreeItem(Item: Pointer);' + CRLF +
        'begin' + CRLF +
        '  if Assigned(Item) then' + CRLF +
        '    T%s(Item).Free;' + CRLF +
        'end;' + CRLF2,
        [Name, ItemType]);
    if SortedCollection then
      FmtWrite(Result,
        'function T%s.Compare(Key1, Key2: Pointer): Longint;' + CRLF +
        'begin' + CRLF +
        '  Result := 0{your code here};' + CRLF +
        'end;' + CRLF2 +
        'function T%0:s.KeyOf(Item: Pointer): Pointer;' + CRLF +
        'begin' + CRLF +
        '  Result := T%s(Item).{your code here};' + CRLF +
        'end;' + CRLF2,
        [Name, ItemType]);
  end;
end;

procedure WriteRegister(C: TClassItem); far;
begin
  with C do
    FmtWrite(Result,
      '  RegisterStreamObject(R%s, OBJTYPE_%0:s, VERSION_%0:s,' + CRLF +
      '    Pointer(T%0:s), @T%0:s.Load, @T%0:s.Store, True_OR_False?);' + CRLF,
      [Name]);
end;

var
  CodeResource: THandle;
  P: PChar;
begin
  SourceBuffer := StrAlloc(SourceBufferSize);
  try
    Result := TMemoryStream.Create;
    try
      { write the interface part }

      { unit header and uses clause }
      FmtWrite(Result,
        'unit %s;' + CRLF2 +
        'interface' + CRLF2 +
        'uses' + CRLF +
        '  {$IFDEF WIN32} Windows,' + CRLF +
        '  {$ELSE}        WinTypes, WinProcs,' + CRLF +
        '  {$ENDIF}       Messages, SysUtils,' + CRLF +
        '  LongColl;' + CRLF2 +
        'const' + CRLF2 +
        '  ID_FIRST_OBJTYPE = $%.4x;' + CRLF2,
        [UnitIdent, edtFirstObject.Value]);

      WorkList.ForEach(@WriteClassInterface);

      { write the implementation part }
      FmtWrite(Result,
        'implementation' + CRLF2 +
        'var' + CRLF,
        [nil]);

      WorkList.ForEach(@WriteStreamRecs);
      FmtWrite(Result,
        'procedure RegisterThisUnit;' + CRLF +
        'begin' + CRLF,
        [nil]);
      WorkList.ForEach(@WriteRegister);
      FmtWrite(Result,
        'end;' + CRLF2,
        [nil]);

      WorkList.ForEach(@WriteFieldImplementation);
      FmtWrite(Result,
        'procedure FreeMemoryAllocatedHere; far;' + CRLF +
        'begin' + CRLF +
        '  { your code here }' + CRLF +
        'end;' + CRLF2 +
        'initialization' + CRLF +
        '  RegisterThisUnit;' + CRLF +
        '  { your code here }' + CRLF +
        '  AddExitProc(FreeMemoryAllocatedHere);' + CRLF +
        'end.' + CRLF2, [nil]);

      CodeResource := LoadResource(HInstance,
        FindResource(HInstance, 'DISCLAIMER', RT_RCDATA));
      try
        P := LockResource(CodeResource);
        FmtWrite(Result, P, [nil]);
      finally
        UnlockResource(CodeResource);
        FreeResource(CodeResource);
      end;
      Result.Position := 0;
    except
      Result.Free;
      raise;
    end;
  finally
    StrDispose(SourceBuffer);
  end;
end;

procedure DialogExpert(ToolServices: TIToolServices);
var
  D: TDlgExpert;
  ISourceStream: TIMemoryStream;
  {$IFDEF WIN32} UnitIdent: string;
                 FileName: String;
  {$ELSE}        UnitIdent: string[79];
                 FileName: TFileName;
  {$ENDIF}
begin
  if ToolServices = nil then Exit;
  if ToolServices.GetNewModuleName(UnitIdent, FileName) then
  begin
    D := TDlgExpert.Create(Application);
    try
      if D.ShowModal = mrOK then
      begin
        UnitIdent := LowerCase(UnitIdent);
        UnitIdent[1] := Upcase(UnitIdent[1]);
        ISourceStream := TIMemoryStream.Create(D.CreateSource(UnitIdent));
        try
	  {$IFDEF WIN32}
	    {$IFNDEF VER120}
	      {$IFNDEF VER130}
		ISourceStream.AddRef;
	      {$ENDIF VER130}
	    {$ENDIF VER120}
	  {$ENDIF WIN32}
	  ToolServices.CreateModule(FileName, ISourceStream, nil,
	    [cmAddToProject, cmShowSource, cmUnNamed, cmMarkModified]);
	finally
	  {$IFNDEF VER120}
	    {$IFNDEF VER130}
	      ISourceStream.OwnStream := True;
	    {$ENDIF VER130}
	  {$ENDIF VER120}
	  ISourceStream.Free;
        end;
      end;
    finally
      D.Free;
    end;
  end;
end;

procedure TDlgExpert.btnfAddClick(Sender: TObject);
var
  D: TdlgField;
  NewType: String;
begin
  D := TdlgField.Create(Self, cbxItemType);
  with D do
  try
    if Sender = btnfEdit then
      with CurrentField do
        SetFields(FieldName, FieldType, FieldPublic);
    if ShowModal = mrOk then
    begin
      if Sender = btnfEdit then
        CurrentClass.Fields.FreeAtIndex(lbx2.ItemIndex);
      if cbxType.ItemIndex = ord(trObject) then
        NewType := cbxObjectName.Text
      else
        NewType := cbxType.Items[cbxType.ItemIndex];
      cbxItemType.Items.Assign(cbxObjectName.Items);
      CurrentClass.AddField(edtName.Text, NewType, chkPublished.Checked);
      Resetlbx2;
    end;
  finally
    Free;
  end;
end;

procedure TDlgExpert.edtClassNameChange(Sender: TObject);
begin
  btnAdd.Enabled := (edtClassName.Text <> '') and
                    (cbxAncestor.ItemIndex <> -1) and
                    ((cbxAncestor.ItemIndex > 4) or
                     (cbxAncestor.ItemIndex in [0,3]) or
                     (cbxItemType.Text <> ''));
  EnableItemsType(cbxAncestor.ItemIndex in [1,2,4]);
end;

procedure TDlgExpert.btnAddClick(Sender: TObject);
var
  AClassItem: TClassitem;
begin
  if lbx1.Items.IndexOf(edtClassName.Text) <> -1 then
    MessageBox(Handle, 'No duplicates please', 'Warning', MB_OK)
  else
  begin
    AClassItem := TClassItem.Create(
      cbxAncestor.Items[cbxAncestor.ItemIndex],
      edtClassName.Text,
      cbxItemType.Text,
      cbxAncestor.ItemIndex in [1,2,4],
      cbxAncestor.ItemIndex in [2,4]);
    AClassItem.Owner := WorkList;
    WorkList.InsertItem(AClassItem);
    lbx1.Items.Add(edtClassName.Text);
    lbx1.ItemIndex := lbx1.Items.IndexOf(edtClassName.Text);
    if cbxItemType.Items.IndexOf(edtClassName.Text) = -1 then
      cbxItemType.Items.Add(edtClassName.Text);
    if cbxAncestor.Items.IndexOf(Format('T%s', [edtClassName.Text])) = -1 then
      cbxAncestor.Items.Add(Format('T%s', [edtClassName.Text]));
    edtClassName.Text := '';
    cbxAncestor.ItemIndex := -1;
    cbxItemType.Text := '';
    lbx1Click(lbx1);
    EnableItemsType(False);
  end;
end;

procedure TDlgExpert.btnDelClick(Sender: TObject);
var
  Found: Integer;
begin
  if MessageBox(Handle, 'Are you sure you want to delete this item?', 'Confirm',
    MB_YESNO or MB_ICONQUESTION) = IDYES then
  begin
    Found := cbxItemType.Items.IndexOf(lbx1.Items[lbx1.ItemIndex]);
    if Found <> -1 then
      cbxItemType.Items.Delete(Found);
    WorkList.FreeAtIndex(lbx1.ItemIndex);
    lbx1.Items.Delete(lbx1.ItemIndex);
    lbx1Click(lbx1);
  end;
end;

procedure TDlgExpert.lbx1Click(Sender: TObject);
begin
  btnDel.Enabled := lbx1.ItemIndex <> -1;
  Resetlbx2;
  btnCreate.Enabled:= lbx1.ItemIndex <> -1;
  btnUp.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex > 0);
  btnDown.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex < lbx1.Items.Count - 1);
end;

procedure TDlgExpert.lbx2Click(Sender: TObject);
begin
  btnfEdit.Enabled := lbx2.ItemIndex <> -1;
  btnfDelete.Enabled := lbx2.ItemIndex <> -1;
end;

function TDlgExpert.CurrentClass: TClassItem;
begin
  if lbx1.ItemIndex = -1 then Result := nil
  else
    Result := TClassItem(WorkList.Items[lbx1.ItemIndex]);
end;

function TDlgExpert.CurrentField: TClassField;
begin
  if (CurrentClass = nil) or (CurrentClass.Fields = nil) then Result := nil
  else
    Result := TClassField(CurrentClass.Fields.Items[lbx2.ItemIndex]);
end;

procedure TDlgExpert.Resetlbx2;

procedure AddIt(O: TClassField); far;
begin
  with O do
    lbx2.Items.Add(FieldName+#9+
                   FieldType+#9+
                   LoadStr(200+Word(FieldPublic)));
end;

begin
  lbx2.Items.Clear;
  lbx2.Clear;
  btnfAdd.Enabled := lbx1.ItemIndex <> -1;
  btnfEdit.Enabled := False;
  btnfDelete.Enabled := False;
  if (CurrentClass = nil) or (CurrentClass.Fields = nil) then Exit;
  CurrentClass.Fields.ForEach(@AddIt);
  lbx2Click(lbx2);
end;

procedure TDlgExpert.btnfDeleteClick(Sender: TObject);
begin
  CurrentClass.Fields.FreeAtIndex(lbx2.ItemIndex);
  Resetlbx2;
end;

procedure TDlgExpert.cbxItemTypeExit(Sender: TObject);
begin
  if cbxItemType.Text <> '' then
    if cbxItemType.Items.IndexOf(cbxItemType.Text) = -1 then
      cbxItemType.Items.Add(cbxItemType.Text);
end;

procedure TDlgExpert.btnSaveClick(Sender: TObject);
var
  S: TDStream;
begin
  if not SaveDialog1.Execute then Exit;
  S := TDStream.Create(SaveDialog1.FileName, fmCreate, 8192);
  try
    WorkList.Store(S);
  finally
    S.Free;
  end;
end;

procedure TDlgExpert.btnLoadClick(Sender: TObject);

procedure Createlbx1(C: TClassItem); far;
begin
  lbx1.Items.Add(C.Name);
end;

var
  S: TDStream;
begin
  if not OpenDialog1.Execute then Exit;
  S := TDStream.Create(OpenDialog1.FileName, fmOpenRead, 8192);
  try
    if Assigned(WorkList) then WorkList.Free;
    WorkList := TClassList.Load(S);
    if Assigned(WorkList) and (WorkList.Count > 0) then
    begin
      lbx1.Clear;
      lbx2.Clear;
      WorkList.ForEach(@Createlbx1);
      lbx1.ItemIndex := 0;
      Resetlbx2;
      btnUp.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex > 0);
      btnDown.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex < lbx1.Items.Count - 1);
      btnCreate.Enabled := True;
    end;
  finally
    S.Free;
  end;
end;

procedure TDlgExpert.UpDownClick(Sender: TObject);
var
  OldIndex, NewIndex: Integer;

procedure ChangeIt(Add: Boolean);
begin
  if Add then Inc(NewIndex)
  else Dec(NewIndex);
  WorkList.Exchange(OldIndex, NewIndex);
  lbx1.Items.Exchange(OldIndex, NewIndex);
  lbx1.ItemIndex := NewIndex;
  btnUp.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex > 0);
  btnDown.Enabled := (lbx1.ItemIndex <> -1) and (lbx1.ItemIndex < lbx1.Items.Count - 1);
end;

begin
  OldIndex := lbx1.ItemIndex;
  NewIndex := OldIndex;
  if OldIndex = -1 then Exit;
  if Sender = btnUp then
    if lbx1.ItemIndex > 0 then ChangeIt(False)
    else
  else
    if lbx1.ItemIndex < lbx1.Items.Count - 1 then ChangeIt(True);
end;

procedure TDlgExpert.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := True;
  if ModalResult = mrOk then
    btnSaveClick(btnSave);
end;

end.

