unit RecTypes;

interface

uses
  LongColl;

type

  TFieldTypes = (trString, trPChar, trChar,
                 trBoolean, trShortint, trSmallint,
                 trLongint, trByte, trWord, trInteger, trDWORD, trCardinal,
                 trReal, trSingle, trDouble, trDateTime, trExtended, trComp,
                 trObject);

const

  ID_FIRST_OBJTYPE = 1;

{ TClassList ------------------------------------------------------ }
const

  OBJTYPE_FieldList = ID_FIRST_OBJTYPE + 0;
  VERSION_FieldList = $0001;

type

  { TFieldList }
  TFieldList = class(TCollection32)
  protected
    function GetStoredItem(S: TDStream): Pointer; override;
    procedure FreeItem(Item: Pointer); override;
  public
    function GetMaxNameLength: Integer;
  end;

{ TClassField ------------------------------------------------------ }
const

  OBJTYPE_ClassField = ID_FIRST_OBJTYPE + 1;
  VERSION_ClassField = $0001;

type

  { TClassField }
  TClassField = class(TDStreamableClass)
  private
    FFieldName : String;
    FFieldType : String;
    FFieldPublic : Boolean;
    FOwner: TFieldList;
    function GetFType: TFieldTypes;
  protected
    constructor Load(S: TDStream); override;
    procedure Store(S: TDStream); override;
  public
    constructor Create(AFieldName, AFieldType: String;
      AFieldPublic: Boolean);
    property FieldName: String read FFieldName write FFieldName;
    property FieldType: String read FFieldType write FFieldType;
    property FieldPublic: Boolean read FFieldPublic write FFieldPublic;
    property FType: TFieldTypes read GetFType;
    property Owner: TFieldList read FOwner;
  end;

{ TClassList ------------------------------------------------------ }
const

  OBJTYPE_ClassList = ID_FIRST_OBJTYPE + 2;
  VERSION_ClassList = $0001;

type

  { TClassList }
  TClassList = class(TCollection32)
  protected
    function GetStoredItem(S: TDStream): Pointer; override;
    procedure FreeItem(Item: Pointer); override;
  end;

{ TClassItem ------------------------------------------------------ }
const

  OBJTYPE_ClassItem = ID_FIRST_OBJTYPE + 3;
  VERSION_ClassItem = $0001;

type

  { TClassItem }
  TClassItem = class(TDStreamableClass)
  private
    FAncestor  : String;
    FName      : String;
    FItemType  : String;
    FFields    : TFieldList;
    FOwner     : TClassList;
    FCollection: Boolean;
    FSortedCollection: Boolean;
  protected
    constructor Load(S: TDStream); override;
    procedure Store(S: TDStream); override;
  public
    constructor Create(AAncestor, AName, AItemType: String;
      ACollection, ASortedCollection: Boolean);
    procedure AddField(AFieldName, AFieldType: String;
      AFieldPublic: Boolean);
    property Ancestor: String read FAncestor write FAncestor;
    property ItemType: String read FItemType write FItemType;
    property Collection: Boolean read FCollection write FCollection;
    property Fields: TFieldList read FFields;
    property Name: String read FName write FName;
    property Owner: TClassList read FOwner write FOwner;
    property SortedCollection: Boolean read FSortedCollection write FSortedCollection;
  end;

implementation

var
  RFieldList,
  RClassList,
  RClassField,
  RClassItem: TStreamRec;

procedure RegisterThisUnit;
begin
  RegisterStreamObject(RFieldList, OBJTYPE_FieldList, VERSION_FieldList,
    Pointer(TFieldList), @TCollection32.Load, @TCollection32.Store, False);

  RegisterStreamObject(RClassField, OBJTYPE_ClassField, VERSION_ClassField,
    Pointer(TClassField), @TClassField.Load, @TClassField.Store, True);

  RegisterStreamObject(RClassList, OBJTYPE_ClassList, VERSION_ClassList,
    Pointer(TClassList), @TCollection32.Load, @TCollection32.Store, False);

  RegisterStreamObject(RClassItem, OBJTYPE_ClassItem, VERSION_ClassItem,
    Pointer(TClassItem), @TClassItem.Load, @TClassItem.Store, True);
end;

{ TClassField }
constructor TClassField.Create(AFieldName, AFieldType: String;
  AFieldPublic: Boolean);
begin
  inherited Create;
  FFieldName   := AFieldName;
  FFieldType   := AFieldType;
  FFieldPublic := AFieldPublic;
end;

constructor TClassField.Load(S: TDStream);
begin
  inherited Load(S);
  FFieldName   := S.ReadStr;
  FFieldType   := S.ReadStr;
  S.Read(FFieldPublic, SizeOf(FFieldPublic));
  LoadDone(TClassField);
end;

procedure TClassField.Store(S: TDStream);
begin
  inherited Store(S);
  S.WriteStr(FFieldName);
  S.WriteStr(FFieldType);
  S.Write(FFieldPublic, SizeOf(FFieldPublic));
  StoreDone(S, TClassField);
end;

function TClassField.GetFType: TFieldTypes;
begin
  if FFieldType = 'String' then Result := trString else
  if FFieldType = 'PChar' then Result := trPChar else
  if FFieldType = 'Char' then Result := trChar else
  if FFieldType = 'Boolean' then Result := trBoolean else
  if FFieldType = 'Shortint' then Result := trShortint else
  if FFieldType = 'Smallint' then Result := trSmallint else
  if FFieldType = 'Longint' then Result := trLongint else
  if FFieldType = 'Byte' then Result := trByte else
  if FFieldType = 'Word' then Result := trWord else
  if FFieldType = 'Integer' then Result := trInteger else
  if FFieldType = 'DWORD' then Result := trDWORD else
  if FFieldType = 'Cardinal' then Result := trCardinal else
  if FFieldType = 'Real' then Result := trReal else
  if FFieldType = 'Single' then Result := trSingle else
  if FFieldType = 'Double' then Result := trDouble else
  if FFieldType = 'TDateTime' then Result := trDateTime else
  if FFieldType = 'Extended' then Result := trExtended else
  if FFieldType = 'Comp' then Result := trComp else
  Result := trObject;
end;

{ TFieldList }
function TFieldList.GetStoredItem(S: TDStream): Pointer;
begin
  Result := inherited GetStoredItem(S);
  if Assigned(Result) then
    TClassField(Result).FOwner := Self;
end;

procedure TFieldList.FreeItem(Item: Pointer);
begin
  if Assigned(Item) then
    TClassField(Item).Free;
end;

function TFieldList.GetMaxNameLength: Integer;

  procedure CheckLength(C: TClassField); far;
  begin
    if Length(C.FFieldName) > Result then
      Result := Length(C.FFieldName); 
  end;

begin
  Result := 0;
  ForEach(@CheckLength);
  Inc(Result, 2);
end;

{ TClassItem }
constructor TClassItem.Create(AAncestor, AName, AItemType: String;
  ACollection, ASortedCollection: Boolean);
begin
  inherited Create;
  FAncestor := AAncestor;
  FCollection := ACollection;
  FItemtype := AItemtype;
  FName     := AName;
  FFields   := nil;
  FSortedCollection := ASortedCollection;
end;

constructor TClassItem.Load(S: TDStream);
begin
  inherited Load(S);
  FAncestor  := S.ReadStr;
  FName      := S.ReadStr;
  FItemType  := S.ReadStr;
  S.Read(FCollection, SizeOf(FCollection));
  S.Read(FSortedCollection, SizeOf(FSortedCollection));
  FFields    := TFieldList(S.Get);
  LoadDone(TClassItem);
end;

procedure TClassItem.Store(S: TDStream);
begin
  inherited Store(S);
  S.WriteStr(FAncestor);
  S.WriteStr(FName);
  S.WriteStr(FItemType);
  S.Write(FCollection, SizeOf(FCollection));
  S.Write(FSortedCollection, SizeOf(FSortedCollection));
  S.Put(FFields);
  StoreDone(S, TClassItem);
end;

procedure TClassItem.AddField(AFieldName, AFieldType: String;
  AFieldPublic: Boolean);
var
  AField: TClassField;
begin
  if not Assigned(FFields) then
    FFields := TFieldList.Create(5, 2);
  AField := TClassField.Create(AFieldName, AFieldType, AFieldPublic);
  AField.FOwner := FFields;
  FFields.InsertItem(AField);
end;

{ TClassList }
function TClassList.GetStoredItem(S: TDStream): Pointer;
begin
  Result := inherited GetStoredItem(S);
  if Assigned(Result) then
    TClassItem(Result).FOwner := Self;
end;

procedure TClassList.FreeItem(Item: Pointer);
begin
  if Assigned(Item) then
    TClassItem(Item).Free;
end;

initialization
  RegisterThisUnit;
end.

