unit GraphObj;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, LongColl;

const

  ID_FIRST_OBJECTTYPE = 150;

{ TGraphObject ------------------------------------------------------ }
const

  ObjectTypeGraphObject = ID_FIRST_OBJECTTYPE + 0;
  VersionGraphObject = {$IFDEF WIN32} $0002;
		       {$ELSE}        $0001;
		       {$ENDIF WIN32}

type

  { TGraphObject }
  TGraphObject = class(TDStreamableClass)
  private
    { Private declarations }
    FRect: TRect;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(ARect: TRect);
    constructor Load(S: TDStream); override;
    procedure Store(S: TDStream); override;
    procedure Draw(DC: HDC); virtual; abstract;

    property TheRect: TRect read FRect write FRect;
  end;

{ TGraphEllipse ------------------------------------------------------ }
const

  ObjectTypeGraphEllipse = ID_FIRST_OBJECTTYPE + 1;
  VersionGraphEllipse = $0001;

type

  { TGraphEllipse }
  TGraphEllipse = class(TGraphObject)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Draw(DC: HDC); override;
  end;

{ TGraphRect ------------------------------------------------------ }
const

  ObjectTypeGraphRect = ID_FIRST_OBJECTTYPE + 2;
  VersionGraphRect = $0001;

type

  { TGraphRect }
  TGraphRect = class(TGraphObject)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Draw(DC: HDC); override;
  end;

{ TGraphPie ------------------------------------------------------ }
const

  ObjectTypeGraphPie = ID_FIRST_OBJECTTYPE + 3;
  VersionGraphPie = {$IFDEF WIN32} $0002;
		    {$ELSE}        $0001;
		    {$ENDIF WIN32}

type

  { TGraphPie }
  TGraphPie = class(TGraphObject)
  private
    { Private declarations }
    FArcStart: TPoint;
    FArcEnd: TPoint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(ARect: TRect);
    procedure Draw(DC: HDC); override;
    constructor Load(S: TDStream); override;
    procedure Store(S: TDStream); override;
  end;

implementation

var
  RGraphObject,
  RGraphEllipse,
  RGraphRect,
  RGraphPie: TStreamRec;

procedure RegisterThisUnit;
begin
  RegisterStreamObject(RGraphObject, ObjectTypeGraphObject, VersionGraphObject,
    Pointer(TGraphObject), @TGraphObject.Load, @TGraphObject.Store, True);
  RegisterStreamObject(RGraphEllipse, ObjectTypeGraphEllipse, VersionGraphEllipse,
    Pointer(TGraphEllipse), @TGraphEllipse.Load, @TGraphEllipse.Store, False);
  RegisterStreamObject(RGraphRect, ObjectTypeGraphRect, VersionGraphRect,
    Pointer(TGraphRect), @TGraphRect.Load, @TGraphRect.Store, False);
  RegisterStreamObject(RGraphPie, ObjectTypeGraphPie, VersionGraphPie,
    Pointer(TGraphPie), @TGraphPie.Load, @TGraphPie.Store, True);
end;

{$IFDEF WIN32}
type
  TsmallPoint = packed record
    X: SmallInt;
    Y: SmallInt;
  end;

  TsmallintRect = packed record
    case SmallInt of
      0: (Left, Top, Right, Bottom: SmallInt);
      1: (TopLeft, BottomRight: SmallInt);
  end;
{$ENDIF WIN32}

{ TGraphObject ------------------------------------------------------ }
constructor TGraphObject.Create(ARect: TRect);
var
  Height, Width: Word;
begin
  inherited Create;
  with ARect do
  begin
    Height := Random(Bottom - Top) div 2 + 10;
    Width := Random(Right - Left) div 3 + 15;
  end;
  with FRect do
  begin
    Left := Random(ARect.Right - ARect.Left - Width);
    Right := Left + Width;
    Top := Random(ARect.Bottom - ARect.Top - Height);
    Bottom := Top + Height;
  end;
end;

constructor TGraphObject.Load(S: TDStream);
{$IFDEF WIN32}
var
  smallRect: TsmallintRect;
{$ENDIF}
begin
  inherited Load(S);
  {$IFDEF WIN32}
  if Version < VersionGraphObject then
  begin
    S.Read(smallRect, SizeOf(smallRect));
    FRect.Left   := smallRect.Left;
    FRect.Top    := smallRect.Top;
    FRect.Right  := smallRect.Right;
    FRect.Bottom := smallRect.Bottom;
  end
  else
  {$ENDIF WIN32}
  S.Read(FRect, SizeOf(FRect));
  LoadDone(TGraphObject);
end;

procedure TGraphObject.Store(S: TDStream);
begin
  inherited Store(S);
  S.Write(FRect, SizeOf(FRect));
  StoreDone(S, TGraphObject);
end;

{ TGraphEllipse ------------------------------------------------------ }
procedure TGraphEllipse.Draw(DC: HDC);
begin
  with TheRect do
    Ellipse(DC, Left, Top, Right, Bottom);
end;

{ TGraphRect ------------------------------------------------------ }
procedure TGraphRect.Draw(DC: HDC);
begin
  with TheRect do
    Rectangle(DC, Left, Top, Right, Bottom);
end;

{ TGraphPie ------------------------------------------------------ }
constructor TGraphPie.Create(ARect: TRect);
var
  Height, Width: Word;
begin
  inherited Create(ARect);
  with ARect do
  begin
    Height := Random(Bottom - Top);
    Width := Random(Right - Left);
    FArcStart.X := Random(Right - Left - Width);
    FArcEnd.X := FArcStart.X + Width;
    FArcStart.Y := Random(Bottom - Top - Height);
    FArcEnd.Y := FArcStart.Y + Height;
  end;
end;

procedure TGraphPie.Draw;
begin
  with TheRect do
    Pie(DC, Left, Top, Right, Bottom,
      FArcStart.X, FArcStart.Y, FArcEnd.X, FArcEnd.Y);
end;

constructor TGraphPie.Load(S: TDStream);
{$IFDEF WIN32}
var
   P: TsmallPoint;
{$ENDIF WIN32}
begin
  inherited Load(S);
  {$IFDEF WIN32}
  if Version < VersionGraphObject then
  begin
    S.Read(P, SizeOf(P));
    FArcStart.X := P.X; FArcStart.Y := P.Y;
    S.Read(P, SizeOf(P));
    FArcEnd.X := P.X; FArcEnd.Y := P.Y;
  end
  else
  {$ENDIF WIN32}
  begin
    S.Read(FArcStart, SizeOf(FArcStart));
    S.Read(FArcEnd, SizeOf(FArcEnd));
  end;
  LoadDone(TGraphPie);
end;

procedure TGraphPie.Store(S: TDStream);
begin
  inherited Store(S);
  S.Write(FArcStart, SizeOf(FArcStart));
  S.Write(FArcEnd, SizeOf(FArcEnd));
  StoreDone(S, TGraphPie);
end;

procedure FreeMemoryAllocatedHere; far;
begin
  { your code here }
end;

initialization
  RegisterThisUnit;
  { your code here }
  AddExitProc(FreeMemoryAllocatedHere);
end.

The information provided above is provided "AS IS" without warranty of any kind.
The AUTHOR disclaims all warranties, either express or implied, including the
warranties of merchantability and fitness for a particular purpose.

In no event shall the AUTHOR or his suppliers be liable for any damages whatsoever
including direct, indirect, incidental, consequential, loss of business profits or
special damages, even if the AUTHOR or his suppliers have been advised of the
possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

